import py.test
import re

from os.path import join
from nose.tools import with_setup
from nose.plugins.attrib import attr
from nose.plugins.skip import SkipTest
from django.template import TemplateSyntaxError
from skylark import *
from skylark.assembly import *
from skylark.page import PageAssembly
from skylark.snippet import SnippetAssembly

from skylark.tests import *


@with_setup(setup, teardown)
def test_bb_issue_07():
    """
    Issue #7

    The problem that this test is verifying is related to the media/js/templates
    directory being copied before other files in the directory.  Our method that
    checks to see if that directory is stale (needs to be updated) was
    malfunctioning.  It compared the dates on the source and cache directory,
    since they were the same (because we just copied the templates directory) it
    was not allowing the rest of the files to be copied.
    """
    request = get_request_fixture()
    c = RequestContext(request, {})
    sa = SnippetAssembly('dummyapp/issue_bb_07/issue_bb_07.yaml', c)

    content = sa.dumps()

    exist(
        'out/dummyapp/issue_bb_07/media/js/Class.js',
        'out/dummyapp/issue_bb_07/media/js/templates/file.html',
    )


@with_setup(setup, teardown)
def test_bb_issue_19():
    """
    Issue #19

    If DEBUG=False, a cached directory may not contain all needed files

    With DEBUG=False, an existing directory will be ignored when creating the
    files within the cfcache.  The reason that this creates a problem, is that
    the directory may be created by another directive like js: -file:
    app/page/media/js/file.js and then another directive may add content to
    that same directory chirp: -location: app/page/media/js. The first js
    will create the directory and the chirp section will then be skipped.
    """
    settings.DEBUG = False
    settings.SKYLARK_PLANS = 'mediadeploy'
    settings.SKYLARK_PLANS_DEFAULT = 'rf'

    request = get_request_fixture()
    c = RequestContext(request, {})
    pa = PageAssembly('dummyapp/issue_bb_19/issue_bb_19.yaml', c)

    content = pa.dumps()

    exist(
        'out/dummyapp/issue_bb_19/media/js/static.js',
        'out/dummyapp/issue_bb_19/media/js/Class.js',
        'out/dummyapp/issue_bb_19/media/js/templates/file.html',
    )


@with_setup(setup, teardown)
def test_bb_issue_23():
    """
    Issue #23

    If you are rendering a page with a PageAssembly and that page contains a
    template tag that renders using SnippetAssembly, the underlying
    context['page_instructions'] object will be changed causing dependencies to
    be missed
    """
    request = get_request_fixture()
    c = RequestContext(request, {})
    pa = PageAssembly('dummyapp/issue_bb_23/issue_bb_23.yaml', c)

    content = pa.dumps()
    stuff = (
        'dummyapp/issue_bb_23/media/js/tt_after_sa.js',
        "dojo.registerModulePath('DummyApp.BB.Issue23.TTSa'",
        "dojo.require('DummyApp.BB.Issue23.TTSa.Controller');",
        "dojo.require('DummyApp.BB.Issue23.TTSa.View');",
        "dojo.registerModulePath('DummyApp.BB.Issue23.TTAfterSa'",
        "dojo.require('DummyApp.BB.Issue23.TTAfterSa.Controller');",
        "dojo.require('DummyApp.BB.Issue23.TTAfterSa.View');",
    )
    for item in stuff:
        assert item in stuff
        assert content.find(item) < content.find("<body>")
        assert content.count(item) == 1


@with_setup(setup, teardown)
def test_bb_issue_24():
    """
    Issue #24

    If the YAML file the SnippetAssembly is rendering does not contain a body, it
    will inherit from the PageAssembly.

    This can cause an infinite loop if there is a templatetag inside the main
    page that is causing the SnippetAssembly to be rendered in the first place.
    """
    request = get_request_fixture()
    c = RequestContext(request, {})
    pa = PageAssembly('dummyapp/issue_bb_24/issue_bb_24.yaml', c)

    py.test.raises(TemplateSyntaxError, pa.dumps)


@with_setup(setup, teardown)
def test_bb_issue_25():
    """
    Issue #25

    Joe, while debugging a dojo.require problem commented out the line but the
    regular expression does not respect commented sections.
    This test needs to include both commenting styles:

    /**
    dojo.require('something')
    */

    // dojo.require('something')
    """
    # TODO Need to still write a test for block comments
    settings.SKYLARK_PLANS = 'mediadeploy_reusable'

    hash_js = 'bbc01a8493372bb2e9127837ed42dfab'

    request = get_request_fixture()
    c = RequestContext(request, {})
    pa = PageAssembly('dummyapp/issue_bb_25/issue_bb_25.yaml', c)

    content = pa.dumps()

    assert content

    jsfile = get_contents(
        os.path.join(cachedir, 'out', '%s.js' % hash_js)
    )

    assert "dojo.registerModulePath('ChirpTools'" in jsfile
    assert "dojo.provide('ChirpTools.Error')" in jsfile
    assert 'dojo.provide("dojox.timing._base")' in jsfile
    assert 'dojo.provide("dojox.timing")' in jsfile
    assert "dojo.provide('ChirpTools.SyncTimer.Timer')" in jsfile
    assert 'dojo.provide("dojo.back")' in jsfile
    assert 'dojo.provide("dojo.NodeList-traverse")' in jsfile
    assert "dojo.registerModulePath('DummyApp.BB.Issue25'" in jsfile
    assert "dojo.provide('DummyApp.BB.Issue25.TestFile')" in jsfile


@with_setup(setup, teardown)
def test_bb_issue_27():
    """
    Issue #27

    Because we tack extra information on a context object (namely the page
    instructions and page assemblies that are involving that context) we are
    forced to use the context that is passed into a render method in a template
    tag.

    This is not ideal, because we may overwrite or clog the context with
    variables that don't belong there.

    The template tag needs to be able to create a new RequestContext based on
    the request and have things work out correctly.

    Behind the scenes, CF needs to do whatever is necessary to keep the page
    instructions and page assemblies hooked to the context. This can be done by
    placing a hook on the request itself.
    """
    request = get_request_fixture()

    assert not hasattr(request, 'skylark_internals')

    from django.core import context_processors
    # Add the request object to our contexts
    processors = (
        context_processors.request,
    )

    context = RequestContext(request,
        { 'foo': 'bar' },
        processors)

    assert context.has_key('skylark_internals')
    assert hasattr(request, 'skylark_internals')

    sa_context = RequestContext(context['request'],
        { 'bob': 'slob' },
        processors)

    assert sa_context['skylark_internals'] is request.skylark_internals
    assert sa_context.has_key('bob')
    # Make sure that we aren't inheriting variables
    assert not sa_context.has_key('foo')

    not_hooked_request = get_request_fixture()
    not_hooked_context = RequestContext(not_hooked_request,
        { 'ted': 'sled' },
        processors)

    # Make sure that we aren't reusing request's skylark internals
    assert not_hooked_context['skylark_internals'] is not \
        request.skylark_internals


@with_setup(setup, teardown)
def test_gh_issue_01():
    settings.SKYLARK_PLANS = 'mediadeploy_reusable'

    request = get_request_fixture()
    c = RequestContext(request, {})
    pa = PageAssembly('dummyapp/issue_gh_01/issue_gh_01.yaml', c)

    content = pa.dumps()
    assert content



import py.test
import re

from os.path import isdir, isfile, join
from nose.tools import with_setup
from nose.plugins.attrib import attr
from nose.plugins.skip import SkipTest
from skylark import *
from skylark.assembly import *
from skylark.page import PageAssembly
from django.http import HttpResponse
from django.template import TemplateDoesNotExist
from yaml.parser import ParserError

from skylark.tests import *


def test_can_not_create_page_assembly():
    py.test.raises(TypeError, "pa = PageAssembly()")
    py.test.raises(TypeError, 'pa = PageAssembly("somefile/test.yaml")')

    """ Make sure that the context is correct """
    c = []
    py.test.raises(ValueError, 'pa = PageAssembly("somefile/test.yaml", c)')

    """ Make sure that you can't send it an empty yaml file """
    request = get_request_fixture()
    c = RequestContext(request, { 'foo': 'bar' })
    py.test.raises(ValueError, 'pa = PageAssembly((), c)')


def test_can_create_page_assembly():
    request = get_request_fixture()
    c = RequestContext(request, { 'foo': 'bar' })
    pa = PageAssembly('somefile/that/doesnt/exist.yaml', c)

    assert isinstance(pa, PageAssembly)

    """ We should be able to pass it a tuple with more than one file in it """
    pa = PageAssembly(('somefile/that/doesnt/exist.yaml', 'someother/file.yaml'), c)


def test_missing_template():
    request = get_request_fixture()
    c = RequestContext(request, { 'foo': 'bar' })
    pa = PageAssembly('somefile/that/doesnt/exist.yaml', c)

    py.test.raises(TemplateDoesNotExist, "pa.get_http_response()")


def test_provides_page_instructions():
    request = get_request_fixture()
    c = RequestContext(request, { 'foo': 'bar' })
    pa = PageAssembly('dummyapp/page/invalid.yaml', c)

    py.test.raises(ParserError, pa.get_http_response)


def test_missing_skylark_settings():
    from skylark.conf import settings

    assert settings.MEDIA_URL
    assert settings.MEDIA_ROOT
    assert settings.SKYLARK_CACHE_ROOT
    assert settings.SKYLARK_CACHE_URL


@with_setup(setup, teardown)
def test_returns_http_response():
    request = get_request_fixture()
    c = RequestContext(request, { 'foo': 'bar' })
    pa = PageAssembly('dummyapp/page/sample.yaml', c)

    assert isinstance(pa.get_http_response(), HttpResponse)


@with_setup(setup, teardown)
def test_include_with_inline():
    request = get_request_fixture()
    c = RequestContext(request, { 'foo': 'bar' })
    pa = PageAssembly('dummyapp/page/includeinline.yaml', c)

    py.test.raises(AttributeError, pa.get_http_response)


@with_setup(setup, teardown)
def test_creates_files_in_cache():
    request = get_request_fixture()
    c = RequestContext(request, { 'foo': 'bar' })
    pa = PageAssembly('dummyapp/page/sample.yaml', c)

    pa.dumps()

    exist(
        'out/dummyapp/page/media/css/sample.css',
        'out/dummyapp/page/media/img/notreferenced.png',
        'out/dummyapp/page/media/img/test.png',
        'out/dummyapp/page/media/js/notreferenced.js',
        'out/dummyapp/page/media/js/sample.js',
        'out/dummyapp/page/media/js/templates/notreferenced.html',
        'out/dummyapp/page/media/js/templates/sample.js',
    )

    from skylark import clear_media_cache, copy_addons
    clear_media_cache()
    assert not isdir(join(cachedir, 'out'))
    # And fix the addons since we just dumped them
    copy_addons()


@with_setup(setup, teardown)
def test_can_render_an_asset():
    request = get_request_fixture()
    c = RequestContext(request, { 'color': 'gray' })
    pa = PageAssembly('dummyapp/page/renderasset.yaml', c)

    content = pa.dumps()

    exist(
        'out/dummyapp/page/media/img/notreferenced.png',
        'out/dummyapp/page/media/img/test.png',
        'out/dummyapp/page/media/js/templates/notreferenced.html',
        'out/dummyapp/page/media/js/templates/sample.js',
    )

    assert 'my_favorite_color = "gray"' in content
    assert 'background-color: gray' in content


@with_setup(setup, teardown)
def test_can_detect_bad_processor():
    request = get_request_fixture()
    c = RequestContext(request, { 'foo': 'bar' })
    pa = PageAssembly('dummyapp/page/badprocessor.yaml', c)

    py.test.raises(AttributeError, pa.get_http_response)


@with_setup(setup, teardown)
def test_can_render_clevercss():
    request = get_request_fixture()
    c = RequestContext(request, { 'foo': 'bar' })
    pa = PageAssembly('dummyapp/page/clevercss.yaml', c)

    pa.dumps()

    media = (cachedir, 'out', 'dummyapp', 'page', 'media')
    css = get_contents(join(*media + ('css', 'clevercss.css')))
    assert css == 'body {\n  background-color: gray;\n}'

    exist(
        'out/dummyapp/page/media/img/notreferenced.png',
        'out/dummyapp/page/media/img/test.png',
        'out/dummyapp/page/media/js/templates/notreferenced.html',
        'out/dummyapp/page/media/js/templates/sample.js',
    )


@with_setup(setup, teardown)
def test_can_render_lesscss():
    request = get_request_fixture()
    c = RequestContext(request, { 'foo': 'bar' })
    pa = PageAssembly('dummyapp/page/lesscss.yaml', c)

    content = pa.dumps()

    assert '<link rel="stylesheet/less"' in content

    media = (cachedir, 'out', 'dummyapp', 'page', 'media')
    css = get_contents(join(*media + ('css', 'lesscss.css')))

    assert '@primary: #252525;' in css
    assert 'color: @primary;' in css


@with_setup(setup, teardown)
def test_missing_yaml_attributes():
    request = get_request_fixture()
    c = RequestContext(request, { 'foo': 'bar' })
    pa = PageAssembly('dummyapp/page/missingbody.yaml', c)

    py.test.raises(AssertionError, pa.get_http_response)

    request = get_request_fixture()
    c = RequestContext(request, { 'foo': 'bar' })
    pa = PageAssembly('dummyapp/page/missingtitle.yaml', c)

    py.test.raises(AssertionError, pa.get_http_response)

    # Combining them should fix our issue
    request = get_request_fixture()
    c = RequestContext(request, { 'foo': 'bar' })
    pa = PageAssembly(('dummyapp/page/missingbody.yaml', 'dummyapp/page/missingtitle.yaml'), c)

    content = pa.dumps()

    assert isinstance(content, unicode), 'The returned value from dumps() did not return a unicode instance, instead the value was %r' % content 


@with_setup(setup, teardown)
def test_will_not_duplicate_assets():
    request = get_request_fixture()
    c = RequestContext(request, { 'foo': 'bar' })
    pa = PageAssembly('dummyapp/page/duplicateassets.yaml', c)

    content = pa.dumps()

    exist(
        'out/dummyapp/page/media/css/sample.css',
        'out/dummyapp/page/media/img/notreferenced.png',
        'out/dummyapp/page/media/img/test.png',
        'out/dummyapp/page/media/js/notreferenced.js',
        'out/dummyapp/page/media/js/sample.js',
        'out/dummyapp/page/media/js/templates/notreferenced.html',
        'out/dummyapp/page/media/js/templates/sample.js',
    )

    assert len(re.findall('sample.js', content)) == 1
    assert len(re.findall('files.js', content)) == 1


@with_setup(setup, teardown)
def test_title_tag_is_escaped():
    request = get_request_fixture()
    c = RequestContext(request, { 'title': unicode('Title < > \' "') })
    pa = PageAssembly('dummyapp/page/escapetitle.yaml', c)

    content = pa.dumps()

    assert content.index('Title &lt; &gt; &#39; &quot;')


@with_setup(setup, teardown)
def test_will_copy_assets():
    request = get_request_fixture()
    c = RequestContext(request, { 'title': unicode('Title < > \' "') })
    pa = PageAssembly('dummyapp/page/sample.yaml', c)

    content = pa.dumps()

    exist(
        'out/dummyapp/page/media/css/sample.css',
        'out/dummyapp/page/media/img/test.png',
        'out/dummyapp/page/media/img/notreferenced.png',
        'out/dummyapp/page/media/js/templates/sample.js',
        'out/dummyapp/page/media/js/notreferenced.js',
        'out/dummyapp/page/media/js/templates/notreferenced.html',
        'out/dummyapp/page/media/js/templates/sample.js',
    )

    assert content.find('notreferenced') == -1, 'Found a reference to a file that has been set include: false in the yaml file.  It should not show up in the rendered output'


@with_setup(setup, teardown)
def test_references_other_yaml_files():
    request = get_request_fixture()
    c = RequestContext(request, { 'background_color': 'red' })
    pa = PageAssembly('dummyapp/page/uses.yaml', c)

    content = pa.dumps()

    exist(
        'out/dummyapp/page/media/css/sample.css',
        'out/dummyapp/page/media/img/notreferenced.png',
        'out/dummyapp/page/media/img/test.png',
        'out/dummyapp/page/media/js/notreferenced.js',
        'out/dummyapp/page/media/js/sample.js',
        'out/dummyapp/page/media/js/sampleafter.js',
        'out/dummyapp/page/media/js/templates/notreferenced.html',
        'out/dummyapp/page/media/js/templates/sample.js',
    )

    assert len(re.findall('.*\.css', content)) == 2, 'There should be 2 css files from the sample.yaml in here'
    assert content.find('sample.js') < content.find('sampleafter.js'), 'The sample.js should come before the sampleafter.js'
    assert "body {" in content
    assert "background-color: red" in content


@with_setup(setup, teardown)
def test_renders_meta_section():
    request = get_request_fixture()
    c = RequestContext(request, { 'foo': 'bar' })
    pa = PageAssembly('dummyapp/page/meta.yaml', c)

    content = pa.dumps()

    exist(
        'out/dummyapp/page/media/img/notreferenced.png',
        'out/dummyapp/page/media/img/test.png',
        'out/dummyapp/page/media/js/templates/notreferenced.html',
        'out/dummyapp/page/media/js/templates/sample.js',
    )

    assert '<meta http-equiv="test" content="test-content">' in content
    assert '<meta name="test-meta" content="foo">' in content


@with_setup(setup, teardown)
def test_will_do_conditional_comments():
    request = get_request_fixture()
    c = RequestContext(request, { 'foo': 'bar' })
    pa = PageAssembly('dummyapp/page/ieversion.yaml', c)

    content = pa.dumps()

    # For the CSS
    assert '<!--[if gte IE 6]>' in content
    # This is for the JS, which we also support
    assert '<!--[if gte IE 7]>' in content


@with_setup(setup, teardown)
def test_will_tidy_output():
    assert settings.DEBUG
    settings.SKYLARK_ENABLE_TIDY = True
    request = get_request_fixture()
    c = RequestContext(request)
    pa = PageAssembly('dummyapp/page/sample.yaml', c)
    assert len(pa.dumps().split("\n")) == 42     # The answer to everything


@with_setup(setup, teardown)
def test_will_use_correct_doctype():
    request = get_request_fixture()
    c = RequestContext(request, { 'foo': 'bar' })
    pa = PageAssembly('dummyapp/page/sample.yaml', c)

    content = pa.dumps()

    assert '<!DOCTYPE html>' in content

    pa = PageAssembly('dummyapp/page/xhtmlstrict.yaml', c)

    content = pa.dumps()

    assert '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"' in content
    assert 'xhtml1-strict.dtd' in content

    exist(
        'out/dummyapp/page/media/css/sample.css',
        'out/dummyapp/page/media/img/notreferenced.png',
        'out/dummyapp/page/media/img/test.png',
        'out/dummyapp/page/media/js/notreferenced.js',
        'out/dummyapp/page/media/js/sample.js',
        'out/dummyapp/page/media/js/templates/notreferenced.html',
        'out/dummyapp/page/media/js/templates/sample.js',
    )


@with_setup(setup, teardown)
def test_will_work_with_templatetags():
    request = get_request_fixture()
    c = RequestContext(request, {})
    pa = PageAssembly('dummyapp/page/tag.yaml', c)

    content = pa.dumps()

    exist(
        'out/dummyapp/page/media/img/notreferenced.png',
        'out/dummyapp/page/media/img/test.png',
        'out/dummyapp/page/media/js/templates/notreferenced.html',
        'out/dummyapp/page/media/js/templates/sample.js',
        'out/dummyapp/tag/media/css/screen.css',
        'out/dummyapp/tag/media/img/testimage.png',
        'out/dummyapp/tag/media/js/templates/test.html',
    )

    assert 'This is my tag test' in content
    assert '/media/cfcache/out/dummyapp/tag/media/css/screen.css" media="screen"' in content


@with_setup(setup, teardown)
def test_bad_html():
    settings.SKYLARK_ENABLE_TIDY = True

    request = get_request_fixture()
    c = RequestContext(request, {})
    pa = PageAssembly('dummyapp/page/badhtml.yaml', c)

    e = py.test.raises(HtmlTidyErrors, pa.dumps)

    assert 'line 25' in str(e.value)
    assert "Warning: <tag> missing '>'" in str(e.value)

    settings.SKYLARK_RAISE_HTML_ERRORS = False

    assert pa.dumps()


@with_setup(setup, teardown)
def test_cache_in_debug_mode():
    request = get_request_fixture()
    c = RequestContext(request, {})
    pa = PageAssembly('dummyapp/page/sample.yaml', c)

    tmp = template.loader.get_template(
        'dummyapp/page/media/js/sample.js')
    sample_path = tmp.origin.name
    temp_path = '%s_temp' % sample_path

    pa.dumps()

    exist(
        'out/dummyapp/page/media/css/sample.css',
        'out/dummyapp/page/media/img/notreferenced.png',
        'out/dummyapp/page/media/img/test.png',
        'out/dummyapp/page/media/js/notreferenced.js',
        'out/dummyapp/page/media/js/sample.js',
        'out/dummyapp/page/media/js/templates/notreferenced.html',
        'out/dummyapp/page/media/js/templates/sample.js',
    )

    content_before = get_contents(join(
        cachedir, 'out', 'dummyapp', 'page', 'media', 'js', 'sample.js')
    )

    # Copy to a temp file and alter the original
    shutil.copyfile(sample_path, temp_path)
    sample_fh = open(sample_path, 'a')
    sample_fh.write('var changed = true;')
    sample_fh.close()

    pa.dumps()
    content_after = get_contents(join(
        cachedir, 'out', 'dummyapp', 'page', 'media', 'js', 'sample.js')
    )

    # Restore the file
    shutil.copyfile(temp_path, sample_path)
    os.remove(temp_path)

    assert content_before != content_after


@with_setup(setup, teardown)
def test_can_register_handlers():
    def handler(page_instructions, renderer, assembly):
        global handler_called
        assembly.add_page_instructions(
            page_instructions, 'dummyapp/page/uses.yaml')
        handler_called = True
        assert page_instructions.other_yaml == ['dummyapp/page/uses.yaml']
        assert page_instructions.uses_yaml == []
        assert page_instructions.root_yaml == 'dummyapp/page/sample.yaml'

    request = get_request_fixture()
    c = RequestContext(request, { 'foo': 'bar' })
    pa = PageAssembly('dummyapp/page/sample.yaml', c)

    pa.register_handler(handler)
    # Make sure we can't get a dupe
    pa.register_handler(handler)

    assert len(BaseAssembly._page_assembly_handlers) == 1

    content = pa.dumps()

    assert handler_called

    pa.unregister_all()

    assert len(BaseAssembly._page_assembly_handlers) == 0


@with_setup(setup, teardown)
def test_can_unregister_handlers():
    def handler(page_instructions, renderer, assembly):
        pass

    request = get_request_fixture()
    c = RequestContext(request, { 'foo': 'bar' })
    pa = PageAssembly('dummyapp/page/sample.yaml', c)

    pa.register_handler(handler)
    pa.unregister_handler(handler)

    assert len(BaseAssembly._page_assembly_handlers) == 0


@with_setup(setup, teardown)
def test_handles_snippets_inside_page_assemblies():
    request = get_request_fixture()
    c = RequestContext(request)
    pa = PageAssembly('dummyapp/page/snippetinside.yaml', c)

    content = pa.dumps()

    assert content.count("DynamicApp.Snippet.Controller") == 1
    assert content.count("dummyapp/snippet/media/js/base.js") == 1
    assert content.count("dummyapp/page/media/js/sample.js") == 1

global handler_called
handler_called = False



from django import template

from skylark.snippet import SnippetAssembly

register = template.Library()


@register.tag()
def tag_uses_snippet(parser, token):
    return TagUsesSnippetNode()


class TagUsesSnippetNode(template.Node):
    def render(self, context):
        sa = SnippetAssembly('dummyapp/snippet/snippet.yaml', context)

        return sa.dumps()



from django import template
from django.template.loader import render_to_string
from skylark.snippet import SnippetAssembly

register = template.Library()


@register.tag
def dummy(parser, token):
    return DummyTagNode()


class DummyTagNode(template.Node):
    def render(self, context):
        sa = SnippetAssembly("dummyapp/tag/tag.yaml", context)
        return sa.dumps()
